////////////////////////////////////////////////////////////////////////////
//
//  TJBDESKTOP  -- This version 1.0 of TJBDesktop component is released
//                 to public domain for free of any usages in any projects,
//                 profit or non-profit.
//
//                 Copyright  1997 by JBed Software.
//                 (All rights are reserved by the owner)
//
//                 If this component is to be used in Business related
//                 projects [software development], a component usage
//                 license is required.  To obtain the license, please
//                 send a self-stamped [addressed] business envelope to:
//
//                      JBed Software
//                      1754  East 16th Street
//                      Brooklyn, NY  11229
//
//                 No tech. support is available for public domain version.
//                 However, general questions may send to:
//
//                      JBLi@MSN.COM
//
////////////////////////////////////////////////////////////////////////////
//
//   This Desktop VCL component can be used to write any [Win95] Desktop
//   related applications or utilities.  The version implements ONLY the
//   sub-set of the functions of the full component [released as 2.0].
//
//   Primary function:  Save and restore Desktop Icon locations.
//
//   Component features:
//
//    -- Types
//
//       TIcondata = This class is used to store display information
//                   for each icon retreived from the desktop by the
//                   SaveIcons() method call.
//
//
//    -- Public Property
//
//       Icons = A String list of retreived information of desktop icons.
//               Each item in the list is pointed to a TIconData Object.
//
//
//    -- Public Methods
//
//       SaveIcons() = This method retreives the display information for
//                     all icons [first level] displayed on the Desktop.
//
//       RestoreIcons() = This method restores the screen coordinates for
//                        all icons [first level] previousely retreived
//                        from the Desktop.
//
//
//
//   VERSION 2.0 FEATURES (not available in this version 1.0)
//
//      - An event is fired for each icon enumerate during SaveIcon method
//        call so the program can decide whether save the icon info or not.
//
//      - Ability to save icons to registry automatically.
//
//      - Ability to restore icons in a given interval.
//
//      - Ability to restrict icon movement in a given interval or names.
//
//      - JBTrayIcon component is included for "Tray Icon" functionality.
//
//      .....
//
////////////////////////////////////////////////////////////////////////////


unit JBDesktop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type


  // This structure is used to store
  // the actual screen coordinate of the icons.
  // (subclass the TObject to be stored in TStrings.)

  TIconData = Class(TObject)
    public
      X : LongInt;
      Y : LongInt;

      procedure SetPos (aX, aY : LongInt);
      function GetPos : TPoint;

  end;



  TJBDesktop = class(TComponent)

  public
    { Public declarations }

    Icons : TStringList;

    constructor Create(aOwner : TComponent); override;
    Destructor Destroy; override;

    function SaveIcons : Boolean;
    function RestoreIcons : Boolean;

  published
    { Published declarations }

  end;

procedure Register;

implementation

uses CommCtrl;



procedure Register;
begin
  RegisterComponents('JBLi', [TJBDesktop]);
end;


// The following ListView related functions are duplicated in order
// to pass parameters to the SysListView32 control.

function JB_ListView_FindItem(hWnd: HWND; iStart: Integer;
                           DisplayText : PChar): Integer;
var  plvfi: PLVFindInfo;
     DisBuf : Pchar;

begin
  try
    plvfi := VirtualAlloc(NIL,Sizeof(TLVFindInfo),$8000000
                          or MEM_COMMIT, PAGE_READWRITE);

    DisBuf := VirtualAlloc(NIL,255,$8000000 or MEM_COMMIT, PAGE_READWRITE);

    StrCopy(DisBuf,DisplayText);
    with plvfi^ do
       begin
         flags:= LVFI_STRING or LVFI_WRAP;
         psz:= DisBuf;
       end;

    Result := SendMessage(hWnd, LVM_FINDITEM, iStart, Longint(plvfi));

  finally
    VirtualFree(plvfi,0,MEM_RELEASE);
    VirtualFree(DisBuf,0,MEM_RELEASE);
  end;

end; // JB_ListView_FindItem




function JB_ListView_GetItemPosition(hwndLV: HWND; i: Integer;
                                     var ppt: TPoint): Bool;
var PointBuf : ^TPoint;

begin
  try
    PointBuf := VirtualAlloc(NIL,Sizeof(TPoint),
                             $8000000 or MEM_COMMIT, PAGE_READWRITE);

    Result := Bool( SendMessage(hWndLV, LVM_GETITEMPOSITION, i, Longint(PointBuf)));
    if Result then
       begin
         ppt.x := PointBuf^.x;
         ppt.y := PointBuf^.y;
       end;

    finally
      VirtualFree(PointBuf,0,MEM_RELEASE);
    end;

end; // JB_ListView_GetItemPosition



function JB_ListView_SetItemText(hwndLV: HWND; i, iSubItem: Integer;
                                 pszText: PChar): Bool;
var Item: PLVItem;

begin
  try
    Item := VirtualAlloc(NIL,Sizeof(TLVItem),
                         $8000000 or MEM_COMMIT, PAGE_READWRITE);
    Item^.iSubItem := 0;
    Item^.pszText := pszText;
    Result := Bool( SendMessage(hwndLV, LVM_SETITEMTEXT, i, Longint(Item)));

  finally
    VirtualFree(Item,0,MEM_RELEASE);
  end;

end;  // JB_ListView_SetItemText


function JB_ListView_GetItemText(hwndLV: HWND; i, iSubItem: Integer;
                                 pszText: PChar; cchTextMax: Integer): Integer;
var  Item: PLVItem;

begin
  try
    Item := VirtualAlloc(NIL,Sizeof(TLVItem),
                         $8000000 or MEM_COMMIT, PAGE_READWRITE);
    Item.iSubItem := iSubItem;
    Item.cchTextMax := cchTextMax;
    Item.pszText := pszText;
    Result := SendMessage(hwndLV, LVM_GETITEMTEXT, i, Longint(Item));

  finally
    VirtualFree(Item,0,MEM_RELEASE);
  end;

end; // JB_ListView_GetItemText





////////////////////////////////////////////////////////////////////////////
// This function retreives the SysListView32 Control
// used by the Win95 Explorer.

function GetSysViewHandle : HWND;
var hWndTemp : HWND;
begin
  Result := GetWindow(GetWindow(
       FindWindow('Progman','Program Manager'),GW_CHILD),GW_CHILD);
end;



////////////////////////////////////////////////////////////////////////////
// Retreive the screen coordinate as a TPoint structure.

function TIconData.GetPos : TPoint;
var P : TPoint;
begin
   P.X := X;
   P.Y := Y;
   Result := P;
end;



////////////////////////////////////////////////////////////////////////////
// Assigns the screen coordinate to the TIconData object.

procedure TIconData.SetPos (aX, aY : LongInt);
begin
  X := aX;
  Y := aY;
end;


////////////////////////////////////////////////////////////////////////////
// Constructor is overrided to create the String List which is used to
// store the information of the icons retreived.

constructor TJBDesktop.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  Icons := TStringList.Create;
  Icons.Sorted := false;
end;


////////////////////////////////////////////////////////////////////////////
// Destructor is overrided to free the String List memory.

Destructor TJBDesktop.Destroy;
var Count : Integer;
begin
  for Count := 0 to Icons.Count -1 do
      Icons.Objects[Count].Destroy;
  Icons.Clear;
  inherited Destroy;
end;


////////////////////////////////////////////////////////////////////////////
// This function saves the desktop icons into the Icon [string] list.

function TJBDesktop.SaveIcons : Boolean;

var HLV        : HWND;
    Buffer     : PChar;
    TotalIcons : Integer;
    Count      : Integer;
    aIcon      : TIconData;
    aPoint     : TPoint;

begin
    Result := False; // Assume failure.

    try
      HLV := GetSysViewHandle;

      // We leave if we can grab the SysView32 handle.
      if (HLV = 0) then Exit;

      // Alocate buffer for retrieve icon data.
      Buffer := VirtualAlloc(NIL,255,$8000000 or MEM_COMMIT, PAGE_READWRITE);
      if (Buffer = Nil) then Exit;

      Icons.Clear;

      //Find out how many Icons needed to be stored.
      TotalIcons := ListView_GetItemCount(HLV);
      if TotalIcons = 0 then
         begin
           Result := TRUE;
           Exit;
         end;

      for Count := 0 to TotalIcons - 1 do
        begin
          Try
            aIcon := TIconData.Create;
            JB_ListView_GetItemText(HLV,Count,0,Buffer,255);
            JB_ListView_GetItemPosition(HLV,Count,aPoint);
            aIcon.SetPos(aPoint.x,aPoint.y);
            Icons.AddObject(StrPas(Buffer),aIcon);
          except
            aIcon.Destroy;
          end;
        end;

    Finally
      VirtualFree(Buffer,0,MEM_RELEASE);
    end;

end;



////////////////////////////////////////////////////////////////////////////
// This function restore the icons previously saved in the Icon [String] list.
// store the information of the icons retreived.

function TJBDesktop.RestoreIcons : Boolean;

var Count, aIndex : Integer;
    HLV           : HWND;

begin
  HLV := GetSysViewHandle;

  // We leave if we can grab the SysView32 handle.
  if (HLV = 0) then Exit;

  for Count := 0 to Icons.Count - 1 do
    begin
      aIndex := JB_ListView_FindItem(HLV,-1,PChar(Icons.Strings[Count]));
      if aIndex >= 0 then
         begin
         with TIconData(Icons.Objects[Count]) do
            ListView_SetItemPosition(HLV,aIndex,X,Y);
         end;
    end;
end;


end.
